/*-------------------------------------------------------------------
Name......... atadrv.h
Author....... Marc Adams
Date......... 24/07/03
Description.. Header file for ata driver
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/


/*************************************************************************
Title........ atadrv.h
Date.........
Author....... Alan Austin
*************************************************************************/

#ifndef HARDWARE_ATADRV
#define HARDWARE_ATADRV

/****	Defines */

#define V_TIME 10000000

/****	Type Definitions */
#define ATA_COMMAND_REGS		(0x12000000)
#define ATA_CONTROL_REGS		(0x12000010)

#define ATA_DATA_OFFSET			0x00

#define ATA_ERROR_OFFSET		0x02	/* 0x04*/
#define ATA_NSECTOR_OFFSET		0x04	/* 0x02*/
#define ATA_SECTOR_OFFSET		0x06

#define ATA_LCYL_OFFSET			0x08	/* 0x08*/
#define ATA_HCYL_OFFSET			0x0a	/* 0x0c*/

#define ATA_SELECT_OFFSET		0x0c	/* 0x0a*/

#define ATA_STATUS_OFFSET		0x0e

#define ATA_FEATURE_OFFSET		ATA_ERROR_OFFSET
#define ATA_COMMAND_OFFSET		ATA_STATUS_OFFSET

#define ATA_DATA_REG			(ATA_COMMAND_REGS+ATA_DATA_OFFSET)
#define ATA_ERROR_REG			(ATA_COMMAND_REGS+ATA_ERROR_OFFSET)
#define ATA_NSECTOR_REG			(ATA_COMMAND_REGS+ATA_NSECTOR_OFFSET)
#define ATA_SECTOR_REG			(ATA_COMMAND_REGS+ATA_SECTOR_OFFSET)
#define ATA_LCYL_REG			(ATA_COMMAND_REGS+ATA_LCYL_OFFSET)
#define ATA_HCYL_REG			(ATA_COMMAND_REGS+ATA_HCYL_OFFSET)
#define ATA_SELECT_REG			(ATA_COMMAND_REGS+ATA_SELECT_OFFSET)
#define ATA_STATUS_REG			(ATA_COMMAND_REGS+ATA_STATUS_OFFSET)
#define ATA_FEATURE_REG			ATA_ERROR_REG
#define ATA_COMMAND_REG			ATA_STATUS_REG


/** ATA register bit masks **/

/* Status Reg */
#define ATA_BSY			0x80
#define ATA_DRDY		0x40
#define ATA_DF			0x20
#define ATA_DSC			0x10
#define ATA_DRQ			0x08
#define ATA_CORR		0x04
#define ATA_CHECK		0x01

/**** hard drive ************/

/* IDE Command Constants */
#define ATA_CMD_RECAL		0x10
#define ATA_CMD_READ		0x20
#define ATA_CMD_WRITE		0x30
#define ATA_CMD_INIT		0x91
#define ATA_CMD_ID			0xEC
#define ATA_CMD_SPINDOWN	0xE0
#define ATA_CMD_SPINUP		0xE1

#define	SECTORTRANSFER		256

typedef enum atasts
{
	ATA_OK,
	RDY_TIMEOUT,
	DRQ_TIMEOUT,
	BSY_TIMEOUT
} ATASTS;

/* ATA drive information block */
typedef struct
{
	UWORD  config;         /* lots of obsolete bit flags */
	UWORD  cyls;           /* "physical" cyls */
	UWORD  reserved2;      /* reserved (word 2) */
	UWORD  heads;          /* "physical" heads */
	UWORD  track_bytes;    /* unformatted bytes per track */
	UWORD  sector_bytes;   /* unformatted bytes per sector */
	UWORD  sectors;        /* "physical" sectors per track */
	UWORD  vendor0;        /* vendor unique */
	UWORD  vendor1;        /* vendor unique */
	UWORD  vendor2;        /* vendor unique */
	UBYTE   serial_no[20];  /* 0 = not_specified */
	UWORD  buf_type;
	UWORD  buf_size;       /* 512 byte increments; 0 = not_specified */
	UWORD  ecc_bytes;      /* for r/w long cmds; 0 = not_specified */
	UBYTE   fw_rev[8];      /* 0 = not_specified */
	UBYTE   model[40];      /* 0 = not_specified */
	UBYTE   max_multsect;   /* 0=not_implemented */
	UBYTE   vendor3;        /* vendor unique */
	UWORD  dword_io;       /* 0=not_implemented; 1=implemented */
	UBYTE   vendor4;        /* vendor unique */
	UBYTE   capability;     /* bits 0:DMA 1:LBA 2:IORDYsw 3:IORDYsup*/
	UWORD  reserved50;     /* reserved (word 50) */
	UBYTE   vendor5;        /* vendor unique */
	UBYTE   tPIO;           /* 0=slow, 1=medium, 2=fast */
	UBYTE   vendor6;        /* vendor unique */
	UBYTE   tDMA;           /* 0=slow, 1=medium, 2=fast */
	UWORD  field_valid;    /* bits 0:cur_ok 1:eide_ok */
	UWORD  cur_cyls;       /* logical cylinders */
	UWORD  cur_heads;      /* logical heads */
	UWORD  cur_sectors;    /* logical sectors per track */
	UWORD  cur_capacity0;  /* logical total sectors on drive */
	UWORD  cur_capacity1;  /*  (2 words, misaligned int)     */
	UBYTE   multsect;       /* current multiple sector count */
	UBYTE   multsect_valid; /* when (bit0==1) multsect is ok */
	unsigned int    lba_capacity;   /* total number of sectors */
	UWORD  dma_1word;      /* single-word dma info */
	UWORD  dma_mword;      /* multiple-word dma info */
	UWORD  eide_pio_modes; /* bits 0:mode3 1:mode4 */
    UWORD  eide_dma_min;   /* min mword dma cycle time (ns) */
    UWORD  eide_dma_time;  /* recommended mword dma cycle time (ns) */
	UWORD  eide_pio;       /* min cycle time (ns), no IORDY  */
	UWORD  eide_pio_iordy; /* min cycle time (ns), with IORDY */
	UWORD  reserved69;     /* reserved (word 69) */
	UWORD  reserved70;     /* reserved (word 70) */
	/* UWORD reservedxx[57];*/     /* reserved (words 71-127) */
	/* UWORD vendor7  [32];*/      /* vendor unique (words 128-159) */
	/* UWORD reservedyy[96];*/     /* reserved (words 160-255) */
} HD_DRIVEID;



typedef union
{
	UBYTE bb[512];
	UWORD bw[256];
	ULONG bl[128];
	HD_DRIVEID info;	/* drive info*/
} HDDATA;

/* macro function definition*/
#define BYTESWAP(x)		((((x)&0x0ff)<<8) | (((x)&0x0ff00)>>8))


typedef struct
{
	WORD	inittimeout;
} ATACFG;

typedef struct atavars
{
	const ATACFG* config;
} ATAVARS;

typedef	struct ataservices
{
	ATASTS	(*ideinit)();
	ATASTS	(*driveid)();
	UBYTE* (*hd_info_model)();
	UBYTE* (*hd_info_fw_rev)();
	UBYTE* (*hd_info_serial_no)();
	UWORD (*hd_info_cyls)();
	UWORD (*hd_info_heads)();
	UWORD (*hd_info_sectors)();
	ATASTS	(*spinup)();
	ATASTS	(*spindown)();
	ATASTS	(*readsectors)(ULONG lba, UWORD *buffer, UBYTE count);
} ATASERVICES;


/****	Prototypes */

#include	"atadrv.p"

#endif
/*************************************************************************/
