 /*************************************************************************
Title........ calypso32.h
Date......... 18 May 2004
Author....... Alan Austin
Description..
***********************************************************************~*/

#ifndef HARDWARE_CALYPSO32
#define HARDWARE_CALYPSO32

/****	Defines */

/* video sync period in ms */
#define	VSYNC_MS				16


/* Memory addressing */


/* video display memory: start address in CREMSON address space */
/* 32 Mbyte memory is fitted */
#define CREMSON_MEMORY_START	((LONG)0x0)

/* display list defines */
/* start address in video memory*/

#define DISPLAY_LIST_START		(LONG)(0x1f40000)

#define NLISTS					(WORD)(1<<2)	/* four display lists - do not change */
#define NLISTS_MASK				(WORD)(NLISTS-1)

/*
Size of each display list in entries.
entry size is long word  (4 bytes)

Calypso 16 has 256K memory reserved for display lists

Therefore: 4 x DISPLAY_LIST_SIZE x NLISTS <= 256x1024
*/
#define DISPLAY_LIST_SIZE 		(LONG)(16*1024)

/* maximum number of simultaneous animations */
#define MAX_ANIMATIONS			((WORD)8)

/* REGISTER addressing. Addresses are offset values from a base address defined in the driver config structure  */

/* Cremson address */
#define CREMSONHOSTBASE			0x1fc0000
#define CREMSONDISPLAYBASE		0x1fd0000
#define CREMSONDRAWBASE			0x1ff0000

/* Cremson Registers */

/* Host Interface Registers */
#define CREMSONDTC				CREMSONHOSTBASE + 0x00	/* DMA Transfer Count Register  */
#define CREMSONDSU				CREMSONHOSTBASE + 0x04	/* DMA Setup Register  */
#define CREMSONDRM				CREMSONHOSTBASE + 0x05	/* DMA Request Mask Register  */
#define CREMSONDST				CREMSONHOSTBASE + 0x06	/* DMA Status Register  */
#define CREMSONDTS				CREMSONHOSTBASE + 0x08	/* DMA Transfer Stop Register  */
#define CREMSONLSTA				CREMSONHOSTBASE + 0x10	/* Display List Transfer Status Register  */
#define CREMSONDRQ				CREMSONHOSTBASE + 0x18	/* DMA Request Register  */
#define CREMSONIST				CREMSONHOSTBASE + 0x20	/* Interrupt Status Register  */
#define CREMSONIMASK			CREMSONHOSTBASE + 0x24	/* Interrupt Mask Register  */
#define CREMSONSRST				CREMSONHOSTBASE + 0x2c	/* Software Reset Register  */
#define CREMSONLSA				CREMSONHOSTBASE + 0x40	/* Display List  Source Address Register  */
#define CREMSONLCO				CREMSONHOSTBASE + 0x44	/* Display List Count Register  */
#define CREMSONLREQ				CREMSONHOSTBASE + 0x48	/* Display List Transfer Request Register  */

/* Graphics Memory Interface Registers */
#define CREMSONMMR				CREMSONHOSTBASE + 0xfffc	/* Memory IF Mode Register  */

/* Display Control Registers */
#define CREMSONDCM				CREMSONDISPLAYBASE + 0x00	/* Display Control Mode Register  */
#define CREMSONDCE				CREMSONDISPLAYBASE + 0x02	/* Display Controller Enable Register  */
#define CREMSONHTP				CREMSONDISPLAYBASE + 0x06	/* Hor Total Pixels Register  */
#define CREMSONHDP				CREMSONDISPLAYBASE + 0x08	/* Hor Display Period Register  */
#define CREMSONHDB				CREMSONDISPLAYBASE + 0x0a	/* Hor Display Boundary Register  */
#define CREMSONHSP				CREMSONDISPLAYBASE + 0x0c	/* HSync Position Register  */
#define CREMSONHSW				CREMSONDISPLAYBASE + 0x0e	/* HSync Width Register  */
#define CREMSONVSW				CREMSONDISPLAYBASE + 0x0f	/* Vsync Width Register  */
#define CREMSONVTR				CREMSONDISPLAYBASE + 0x12	/* Vert Total Rasters Register  */
#define CREMSONVSP				CREMSONDISPLAYBASE + 0x14	/* Vsync Position Register  */
#define CREMSONVDP				CREMSONDISPLAYBASE + 0x16	/* Vert Display Period Register  */
#define CREMSONWX				CREMSONDISPLAYBASE + 0x18	/* Window Position X Register  */
#define CREMSONWY				CREMSONDISPLAYBASE + 0x1a	/* Window Position Y Register  */
#define CREMSONWW				CREMSONDISPLAYBASE + 0x1c	/* Window Width Register  */
#define CREMSONWH				CREMSONDISPLAYBASE + 0x1e	/* Window Height Register  */
#define CREMSONCM				CREMSONDISPLAYBASE + 0x20	/* C Layer Mode */
#define CREMSONCOA				CREMSONDISPLAYBASE + 0x24	/* C Layer Origin Address */
#define CREMSONCDA				CREMSONDISPLAYBASE + 0x28	/* C Layer Display Address  */
#define CREMSONCDX				CREMSONDISPLAYBASE + 0x2c	/* C Layer Display Position X */
#define CREMSONCDY				CREMSONDISPLAYBASE + 0x2e	/* C Layer Display Position Y */
#define CREMSONWM				CREMSONDISPLAYBASE + 0x30	/* W Layer Mode */
#define CREMSONWOA				CREMSONDISPLAYBASE + 0x34	/* W Layer Origin Address */
#define CREMSONWDA				CREMSONDISPLAYBASE + 0x38	/* W Layer Display Address */
#define CREMSONMLM				CREMSONDISPLAYBASE + 0x40	/* ML Layer Mode				*/
#define CREMSONMLOA0			CREMSONDISPLAYBASE + 0x44	/* ML Layer Origin Address 0	*/
#define CREMSONMLDA0			CREMSONDISPLAYBASE + 0x48	/* ML Layer Display Address 0	*/
#define CREMSONMLOA1			CREMSONDISPLAYBASE + 0x4c	/* ML Layer Origin Address 1	*/
#define CREMSONMLDA1			CREMSONDISPLAYBASE + 0x50	/* ML Layer Display Address 1	*/
#define CREMSONMLDX				CREMSONDISPLAYBASE + 0x54	/* ML Layer Display Position X	*/
#define CREMSONMLDY				CREMSONDISPLAYBASE + 0x56	/* ML Layer Display Position Y	*/
#define CREMSONMRM				CREMSONDISPLAYBASE + 0x58	/* MR Layer Mode				*/
#define CREMSONMROA0			CREMSONDISPLAYBASE + 0x5c	/* MR Layer Origin Address 0	*/
#define CREMSONMRDA0			CREMSONDISPLAYBASE + 0x60	/* MR Layer Display Address 0	*/
#define CREMSONMROA1			CREMSONDISPLAYBASE + 0x64	/* MR Layer Origin Address 1	*/
#define CREMSONMRDA1			CREMSONDISPLAYBASE + 0x68	/* MR Layer Display Address 1	*/
#define CREMSONMRDX				CREMSONDISPLAYBASE + 0x6c	/* MR Layer Display Position X	*/
#define CREMSONMRDY				CREMSONDISPLAYBASE + 0x6e	/* MR Layer Display Position Y	*/
#define CREMSONBLM				CREMSONDISPLAYBASE + 0x70	/* BL Layer Mode				*/
#define CREMSONBLOA0			CREMSONDISPLAYBASE + 0x74	/* BL Layer Origin Address 0	*/
#define CREMSONBLDA0			CREMSONDISPLAYBASE + 0x78	/* BL Layer Display Address 0	*/
#define CREMSONBLOA1			CREMSONDISPLAYBASE + 0x7c	/* BL Layer Origin Address 1	*/
#define CREMSONBLDA1			CREMSONDISPLAYBASE + 0x80	/* BL Layer Display Address 1	*/
#define CREMSONBLDX				CREMSONDISPLAYBASE + 0x84	/* BL Layer Display Position X	*/
#define CREMSONBLDY				CREMSONDISPLAYBASE + 0x86	/* BL Layer Display Position Y	*/
#define CREMSONBRM				CREMSONDISPLAYBASE + 0x88	/* BR Layer Mode				*/
#define CREMSONBROA0			CREMSONDISPLAYBASE + 0x8c	/* BR Layer Origin Address 0	*/
#define CREMSONBRDA0			CREMSONDISPLAYBASE + 0x90	/* BR Layer Display Address 0	*/
#define CREMSONBROA1			CREMSONDISPLAYBASE + 0x94	/* BR Layer Origin Address 1	*/
#define CREMSONBRDA1			CREMSONDISPLAYBASE + 0x98	/* BR Layer Display Address 1	*/
#define CREMSONBRDX				CREMSONDISPLAYBASE + 0x9c	/* BR Layer Display Position X	*/
#define CREMSONBRDY				CREMSONDISPLAYBASE + 0x9e	/* BR Layer Display Position Y	*/
#define CREMSONBRATIO			CREMSONDISPLAYBASE + 0xb4	/* C Layer Blend Ratio Control */
#define CREMSONBMODE			CREMSONDISPLAYBASE + 0xb6	/* C Layer Blend Mode Control */
#define CREMSONCTC				CREMSONDISPLAYBASE + 0xbc	/* C Layer Transparent Control */
#define CREMSONMRTC				CREMSONDISPLAYBASE + 0xc0	/* MR Layer Transparent Control */
#define CREMSONMLTC				CREMSONDISPLAYBASE + 0xc2	/* ML Layer Transparent Control */


#define CREMSONCPAL				CREMSONDISPLAYBASE + 0x400	/* C Layer Palette Base */
#define CREMSONMBPAL			CREMSONDISPLAYBASE + 0x800	/* C Layer Palette Base */

/* Draw Control Registers */
#define CREMSONCTR				CREMSONDRAWBASE + 0x400		/* Control */

/* Draw Mode Parameter Registers accessed via SetRegister Display List Command */
#define CREMSONMDR0				0x420						/* Misc Mode Register */
#define CREMSONMDR1				0x424						/* Mode Register for LINE*/
#define CREMSONMDR4				0x430						/* BLT Mode Register */
#define CREMSONFBR				0x440						/* Drawing Frame Base Register */
#define CREMSONXRES				0x444						/* Drawing Frame Base Register */
#define CREMSONFC				0x480						/* Foreground Color */
#define CREMSONBC				0x484						/* Background Color */

#define FAST2DLINEREG			0x540						/* 2d line Draw register */

/*
		MDR1 default value:
 		compare mode disable
		Never compare Z coordoinate
		ZWRITMASK set to zero
		Blend mode set to copy
		LOGICAL OPERATION COPY mode
		Solid lines
		Line width =1 pixel
*/
#define MDR1_DEFAULT_VALUE		0x00000640

/* Display Control Mode Register - External Sync enable bit ESY=1, Sync format hi active SF=1*/
#define SYNC_LOCK				((BYTE)0x0C)

/* register index values for initialisation tables */
enum
{
	HOR_SIZE,
	VER_SIZE,
	DCM,
	HTP,
	HDP,
	HDB,
	HSP,
	HSW,
	VSW,
	VTR,
	VSP,
	VDP
};

/* Display List Commands */

#define SETREGISTER				0xF1
#define BLTCOPYALTP     		0x0F
#define DRAWBITMAPP     		0x0B
#define INTERRUPT				0xFD
#define NOP						0xFF
#define DRAWRECTP				0x09
#define DRAWLINE2IP				0x04

#define	VERTEX0					0x0
#define	VERTEX1					0x1
#define	VERTEX2					0x2
#define	VERTEX_INHIBITED		0x3

#define NORMAL_COMMAND			0xff
#define ZERO_VECTOR				0x30
#define ANTI_ZERO_VECTOR		0x38


#define TOPLEFT         		0x44
#define BLTDRAW         		0x42
#define BITMAP          		0x43
#define BLTFILL         		0x41

/* buffer modes*/
#define SINGLEBUFFER			0
#define DOUBLEBUFFER			1

/* layers */
#define C_LAYER					0x00
#define W_LAYER					0x01
#define ML_LAYER				0x02
#define MR_LAYER				0x03
#define BL_LAYER				0x04
#define BR_LAYER				0x05

/* layer selects - old style */
#define CONSOLE_ENABLE			0x01
#define WINDOW_ENABLE			0x02
#define MIDDLE_ENABLE			0x04
#define BASE_ENABLE				0x08
#define ALL_ENABLE				0x0f

/* layer selects - new style */
#define CONSOLE_SELECT			0x01
#define WINDOW_SELECT			0x02
#define MIDDLE_SELECT			0x04
#define BASE_SELECT				0x08
#define ALL_SELECT				0x0f

/* double buffer frames */
#define DOUBLEBUFFER_FRAME0		0
#define DOUBLEBUFFER_FRAME1		1
#define	DOUBLEBUFFER_AUTO		2

/* total number of display layers */
#define NLAYERS					6

/* palette select */
#define CONSOLE_LAYER_PALETTE		0
#define MIDDLE_BASE_LAYER_PALETTE	1


/* color modes */
#define INDIRECT_COLOR			0
#define DIRECT_COLOR			1


/* update sprite mode */
#define NORMAL_MODE 			0
#define TRANSPARENT_MODE 		0x08


/* BLT ROP function definitions */
#define BLT_CLEAR				((LONG)0x0100)
#define BLT_COPY				((LONG)0x0600)
#define BLT_AND					((LONG)0x0300)
#define BLT_OR					((LONG)0x0f00)


/* standard vga palette colours */
#define	BLACK					0
#define	BLUE					1
#define	GREEN					2
#define	CYAN					3
#define	RED						4
#define	MAGENTA					5
#define	BROWN					6
#define	LIGHTGRAY				7
#define	DARKGRAY				8
#define	LIGHTBLUE				9
#define	LIGHTGREEN				10
#define	LIGHTCYAN				11
#define	LIGHTRED				12
#define	LIGHTMAGENTA			13
#define	YELLOW					14
#define	WHITE					15

#define TRANSPARENT				255
#define	NORMAL					0

#define PALETTE_SIZE 			256

/****	Type Definitions */

/* palette entries */
typedef	struct palette_rgb
{
	UBYTE	alpha;
	UBYTE	red;
	UBYTE	green;
	UBYTE	blue;
} PALETTE_RGB;

/* vertical frame variables */
typedef	struct	framevars
{
	BYTE framehz;		/* vertical frame rate - enumerated list below */
	BYTE framecount;	/* count up to framehz */
	BYTE frameflag;		/* frame flip flag */
	WORD framenumber;	/* increments every frame */
	BYTE db_frame;		/* currently displayed double buffer frame 0 or 1 */
	BYTE db_enable;		/* double buffering enable - if enabled flip at vertical frame rate */
	UBYTE vsync_ms_count;	/* time in ms since last vertical interrupt */
} FRAMEVARS;

enum
{
	FRAME_0HZ,
	FRAME_60HZ,
	FRAME_30HZ,
	FRAME_20HZ,
	FRAME_15HZ,
	FRAME_12HZ,
	FRAME_10HZ,
	FRAME_5HZ = 12,
	FRAME_3HZ = 20,
	FRAME_2HZ = 30,
	FRAME_1HZ = 60
};


/* display list */
typedef struct displaylist
{
	LONG count;
	LONG* param;
} DISPLAYLIST;

typedef struct calypsolayer
{
	LONG mode;				/* SINGLEBUFFER | DOUBLEBUFFER */
	BYTE color_mode;		/* INDIRECT_COLOR | DIRECT_COLOR 	8bit / 16bit */
	WORD transparent_color;
	WORD width;	    		/* layer width in pixels */
	WORD height;			/* layer height in lines */
	WORD xoffset;			/* offset from top left to start address: x coordinate in pixels  */
	WORD yoffset;			/* offset from top left to start address: y coordinate in lines   */
	LONG orig_adr0;			/* start address double buffer frame 0 */
	LONG orig_adr1;			/* start address double buffer frame 1 */
	LONG Xpos;				/* drawing position: x coordinate in pixels */
	LONG Ypos;				/* drawing position: y coordinate in lines  */
} CALYPSOLAYER;



typedef struct calypsoimage
{
	ULONG size_bytes;			/* image size bytes */
	WORD color8;				/* 1=8bit 0=16bit */
	WORD palette_size;			/* number of palette entries */
	WORD width;					/* width in pixels */
	WORD height;				/* height in lines */
	const PALETTE_RGB *palette;	/* palette data */
	const WORD *data;			/* image data */
} CALYPSOIMAGE;


/*** Animation control ***/

/* animation types */
enum
{
	ANIMATION_TYPE1,	/* update position --> put sprite */
	ANIMATION_TYPE2,	/* clear area --> update position --> put sprite */
	ANIMATION_TYPE3		/* restore background --> update position --> save background -->put sprite */
};


/* animation run control bit fields */
enum
{
	ANIMATION_STOP,
	ANIMATION_RUN,
	ANIMATION_PAUSE,
	ANIMATION_SINGLE_STEP
};


#define ANIMATION_RUN_FOREVER	0


typedef struct animation
{
	LONG layer_select;		/* display layer */
	UWORD n_frames;			/* total number of frames in animation. ( nframes=0 - run forever ) */
	UWORD current_frame;	/* current frame. Increments each frame. Animation stops when current_frame = n_frames */
	WORD start_x;			/* start position x */
	WORD start_y;			/* start position y */
	WORD start_dx;			/* start velocity x */
	WORD start_dy;			/* start velocity y */
	WORD x;					/* current position x */
	WORD y;					/* current position y */
	WORD dx;				/* current velocity x */
	WORD dy;				/* current velocity y */
	WORD save_x[2];			/* saved x [double buffer frame] (equals -1 if no value saved) */
	WORD save_y[2];			/* saved y [double buffer frame] */
	LONG image_start;		/* image: video memory start address */
	LONG image_offset;		/* multiple image sequence: video memory offset between consecutive images */
	WORD n_images;			/* multiple image sequence: number of images */
	WORD image_index;		/* multiple image sequence: current image index */
	LONG width;				/* image: width in pixels */
	LONG height;			/* image: height in lines */
	BYTE type;				/* ANIMATION_TYPE1, ANIMATION_TYPE2, ANIMATION_TYPE3 */
	LONG mode;				/* Blit mode: NORMAL_MODE or TRANSPARENT_MODE */
	BYTE run_control;		/* ANIMATION_STOP, ANIMATION_RUN, ANIMATION_PAUSE, ANIMATION_SINGLE_STEP */
	BYTE frame_advance;		/* If run_control=ANIMATION_SINGLE_STEP, advance to next frame when frame_advance true */
	BYTE state;				/* state variable */
	LONG save_buffer[2];	/* save buffer addresses for saving background (type = ANIMATION_TYPE3) */
	void (*update)(struct animation* anim);
							/* pointer to user supplied update function - calculates next x,y */
} ANIMATION;

/* nodes for linked list used for animations */
typedef struct node
{
	ANIMATION* anim;		/* each node defines a single animation */
	struct node *next;		/* pointer to next node */
} NODE;

/* animation linked list */
typedef struct animationlist
{
	NODE *nodes;
	NODE *head;
} ANIMATIONLIST;


/* character fonts */

/* Unicode character handling: 512 font pages of 128 characters */
#define PAGE_SIZE				128
#define NPAGES					512

typedef struct calypsochar
{
	UWORD 		offset;
	UBYTE 		dx;
	UBYTE 		dy;
	UBYTE 		width;
	UBYTE 		height;
	BYTE 		xoff;
	BYTE 		yoff;

} CALYPSOCHAR;

typedef struct calypsofont
{
	UWORD 		code_page;
	UBYTE 		width;
	UBYTE 		height;
	BYTE 		xoff;
	BYTE 		yoff;
	CALYPSOCHAR char_metrics[PAGE_SIZE];
	const UBYTE *bitmaps;

} CALYPSOFONT;

/* calypso config structure */
typedef	struct calypso32pluscfg
{
	const WORD* screenmode;
	const PALETTE_RGB*	palette;
	LONG base_address;
	void (*isr)();
	void (**fnc_lst)();
	void (*isr1ms)();

} CALYPSO32PLUSCFG;


/* calypso vars structure */
typedef	struct	calypso32plusvars
{
	ULONG base_address;
	FRAMEVARS* framevars;
	CALYPSOLAYER *layer;
	DISPLAYLIST *display_list_array;
	UBYTE current_display_list;
	ANIMATIONLIST *animation_list;
	BYTE animation_list_lock;
	WORD hsize;
	WORD vsize;

} CALYPSO32PLUSVARS;

/* calypso driver services */
typedef	struct calypso32services
{
	void (*calypsoinitdisplaymode)(const DEVICE *dev, const WORD *reg_table);
	void (*calypsovideoon)(const DEVICE *dev);
	void (*calypsovideooff)(const DEVICE *dev);
	UBYTE (*calypsogetvsync)(const DEVICE *dev);
	ULONG (*calypsosetvideomemory)(const DEVICE *dev, LONG video_addr, LONG size_bytes, BYTE color_mode, UWORD color);
	LONG (*calypsoinitlayer)(const DEVICE *dev, LONG layer_select, BYTE color_mode, BYTE buffer_mode, WORD transparent_color, WORD width_pixels, WORD height, LONG start_address);
	void (*calypsoenablelayer)(const DEVICE *dev, UBYTE select);
	void (*calypsodisablelayer)(const DEVICE *dev, UBYTE select);
	void (*calypsoscrolllayerx)(const DEVICE *dev, LONG layer_select, ULONG offset);
	void (*calypsoscrolllayery)(const DEVICE *dev, LONG layer_select, ULONG offset);
	WORD (*calypsoclearlayer)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame);
	WORD (*calypsofilllayer)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, UWORD color, LONG mode);
	void (*calypsosetwindowlayer)(const DEVICE *dev, WORD x_position, WORD y_position, WORD width, WORD height);
	void (*calypsosetframerate)(const DEVICE *dev, BYTE rate);
	void (*calypsoenabledoublebuffer)(const DEVICE *dev);
	void (*calypsodisabledoublebuffer)(const DEVICE *dev);
	void (*calypsosetdoublebufferframe)(const DEVICE *dev, BYTE frame);
	BYTE (*calypsogetdoublebufferframe)(const DEVICE *dev);
	void (*calypsosetframenumber)(const DEVICE *dev, WORD n);
	WORD (*calypsogetframenumber)(const DEVICE *dev);
	void (*calypsoframeadvance)(const DEVICE *dev);
	void (*calypsoscreenpartitionx)(const DEVICE *dev, WORD partition_xpos);
	void (*calypsoalphablend)(const DEVICE *dev, BYTE enable);
	void (*calypsosetblendratio)(const DEVICE *dev, BYTE blend_ratio, BYTE select);
	void (*calypsoloadpalette)(const DEVICE *dev, UBYTE palette_select, const PALETTE_RGB *pal_table);
	void (*calypsogetpaletteentry)(const DEVICE *dev, UBYTE palette_select, UBYTE entry, PALETTE_RGB *rgb);
	void (*calypsosetpaletteentry)(const DEVICE *dev, UBYTE palette_select, UBYTE entry, PALETTE_RGB *col);
	LONG (*calypsoloadimage)(const DEVICE *dev, LONG video_addr, const CALYPSOIMAGE *image);
	LONG (*calypsoloadimageconvert8to16)(const DEVICE *dev, LONG video_addr, const CALYPSOIMAGE *image);
	LONG (*calypsoloadimagelayer)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, const CALYPSOIMAGE *image);
	LONG (*calypsoloadimagelayerconvert8to16)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, const CALYPSOIMAGE *image);
	LONG (*calypsoloadsprite)(const DEVICE *dev, LONG video_addr, const CALYPSOIMAGE *image);
	WORD (*calypsoputsprite)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, LONG x, LONG y, LONG image_start, LONG xsize, LONG ysize, LONG mode);
	WORD (*calypsosavesprite)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, LONG x, LONG y, LONG save_address, LONG xsize, LONG ysize);
	WORD (*calypsoclearsprite)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, LONG x, LONG y, LONG xsize, LONG ysize);
	WORD (*calypsofillsprite)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, LONG x, LONG y, LONG xsize, LONG ysize, UWORD color, LONG mode);
	void (*calypsosetasciifont)(const DEVICE *dev, const CALYPSOFONT *font);
	void (*calypsosetunicodefont)(const DEVICE *dev, const CALYPSOFONT *font, UWORD unicode_page);
	CALYPSOFONT* (*calypsogetunicodefont)(const DEVICE *dev, UWORD unicode_page);
	WORD (*calypsosettextcolor)(const DEVICE *dev, UWORD foreground_color, UWORD background_color);
	void (*calypsosettextposition)(const DEVICE *dev, LONG layer_select, WORD x, WORD y);
	WORD (*calypsogettextpositionx)(const DEVICE *dev, LONG layer_select);
	WORD (*calypsogettextpositiony)(const DEVICE *dev, LONG layer_select);
	WORD (*calypsoputchar)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, UBYTE ch);
	WORD (*calypsoputstr)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, BYTE *str);
	WORD (*calypsoputunicodechar)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, UWORD ch);
	WORD (*calypsoputunicodestr)(const DEVICE *dev, LONG layer_select, BYTE doublebuffer_frame, UWORD *str);
	void (*calypsopoint)(const DEVICE *dev, LONG layer_select, LONG x, LONG y, UWORD color);
	void (*calypsoline)(const DEVICE *dev, LONG layer_select, LONG Ax, LONG Ay, LONG Bx, LONG By, UWORD color);
	void (*calypsocircle)(const DEVICE *dev, LONG layer_select, LONG xcentre, LONG ycentre, LONG radius, UBYTE color);
	void (*calypsodefineanimation)(const DEVICE *dev, ANIMATION *anim, LONG layer_select, UWORD n_frames, WORD x, WORD y, WORD dx, WORD dy, LONG image_start, LONG image_offset, WORD n_images, WORD image_index, LONG width, LONG height, BYTE type, LONG mode, LONG buffer1, LONG buffer2, void (*update)(ANIMATION *anim));
	void (*calypsoclearanimationlist)(const DEVICE *dev);
	WORD (*calypsoaddtoanimationlist)(const DEVICE *dev, ANIMATION *anim);
	WORD (*calypsoremovefromanimationlist)(const DEVICE *dev, ANIMATION *anim);
	void (*calypsostartanimation)(const DEVICE *dev, ANIMATION *anim);
	void (*calypsostopanimation)(const DEVICE *dev, ANIMATION *anim);
	void (*calypsopauseanimation)(const DEVICE *dev, ANIMATION *anim);
	void (*calypsostepanimation)(const DEVICE *dev, ANIMATION *anim);
	WORD (*calypsogetdisplayhsize)(const DEVICE *dev);
	WORD (*calypsogetdisplayvsize)(const DEVICE *dev);
	void (*calypsosynclockenable)(const DEVICE *dev, BYTE enable);
	WORD (*calypsosetlineproperties)(const DEVICE* dev, UBYTE width);
	WORD (*calypsoadvanceline)(const DEVICE* dev, LONG layer_select, BYTE doublebuffer_frame,LONG Ax, LONG Ay, LONG Bx, LONG By,UWORD color);
	WORD (*calypsoadvancerectangle)(const DEVICE* dev, LONG layer_select, BYTE doublebuffer_frame,LONG left, LONG top, LONG right, LONG bottom, UWORD color);
	void (*calypsoadvancesetlineantialias)(void);
	void (*calypsoadvanceclearlineantialias)(void);
} CALYPSO32SERVICES;



/****	Prototypes */

#include	"calypso32.p"

#endif
/*************************************************************************/
