/*-------------------------------------------------------------------
Name......... mxidev.h
Author....... Marc Adams
Date......... 28/07/03
Description.. Header file for the multiplexed inputs driver
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

#ifndef HARDWARE_MXIDEV
#define HARDWARE_MXIDEV

/****	Defines */

	#define		EOF_INPUTS	-1

/*	Multiplexed or Direct */
	#define		DIR			0x8000
	#define		DEB			0x0000

/*	Active High or Active Low */
	#define		ACT0		0x0000
	#define		ACT1		0x4000

/*	Dummy TRUE or Dummy FALSE */
	#define		IPTRUE		(0x2000 | TRUE)
	#define		IPFALSE		(0x2000 | FALSE)

/*	Multiplexed inputs */
	#define		MXI			0x1000
	#define		STRMSK		0x0F00
	#define		STR0		0x0000
	#define		STR1		0x0100
	#define		STR2		0x0200
	#define		STR3		0x0300

	#define		MXI0	(MXI | STR0)
	#define		MXI1	(MXI | STR1)
	#define		MXI2	(MXI | STR2)
	#define		MXI3	(MXI | STR3)

/* 	Inputs Definitions */
/*	OP0 of Long */
	#define		IP0	24
	#define		IP1	25
	#define		IP2	26
	#define		IP3	27
	#define		IP4	28
	#define		IP5	29
	#define		IP6	30
	#define		IP7	31

/*	OP1 of Long */
	#define		IP8	16
	#define		IP9	17
	#define		IP10	18
	#define		IP11	19
	#define		IP12	20
	#define		IP13	21
	#define		IP14	22
	#define		IP15	23

/*	OP2 of Long */
	#define		IP16	8
	#define		IP17	9
	#define		IP18	10
	#define		IP19	11
	#define		IP20	12
	#define		IP21	13
	#define		IP22	14
	#define		IP23	15

/*	OP3 of Long */
	#define		IP24	0
	#define		IP25	1
	#define		IP26	2
	#define		IP27	3
	#define		IP28	4
	#define		IP29	5
	#define		IP30	6
	#define		IP31	7

/****	Type Definitions */

typedef	struct inputmap
{
	LONG	map0;
} INPUTMAP;

typedef struct	inputcfg
{
	BYTE	debouncesampletime;
} INPUTCFG;

typedef	struct inputvars
{
	INPUTMAP	inputmap;
	INPUTMAP	debarray[2];
	const INPUTCFG*		config;
	volatile BYTE		valid;
} INPUTVARS;

typedef	WORD INPUT;

typedef struct inputservices
{
	BYTE	(*test)(const INPUT ip);
	BYTE	(*testdeb)(const INPUT ip);
} INPUTSERVICES;

/****	Prototypes */

	#include	"mxidev.p"

#endif
/********************************************************************/
