/*-------------------------------------------------------------------
Name......... video.h
Author....... Marc Adams
Date......... 04/11/03
Description.. This file contains the definitions and data types for the
............. Pluto 6 video driver based on the Fujitsu Cremson graphics
............. processor.  General graphics defines and types are listed
............. at the start of the file.
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

#ifndef HARDWARE_VIDEO
#define HARDWARE_VIDEO

/*** GENERAL GRAPHICS DECLARATIONS ***/

/****	Defines */
/* 	Text colours
	NOTE: Only valid if standard VGA palette loaded.
*/
#define	 BLACK				0
#define	 BLUE				1
#define	 GREEN				2
#define	 CYAN				3
#define	 RED				4
#define	 MAGENTA			5
#define	 BROWN				6
#define	 LIGHTGRAY			7
#define	 DARKGRAY			8
#define	 LIGHTBLUE			9
#define	 LIGHTGREEN			10
#define	 LIGHTCYAN			11
#define	 LIGHTRED			12
#define	 LIGHTMAGENTA		13
#define	 YELLOW				14
#define	 WHITE				15

#define TRANSPARENT			255
#define	NORMAL				0

#define	MAXCHARACTERHEIGHT	32
#define	MAXCHARACTERWIDTH	32
	/* character width and height restricted to 32 pixels */

/****	Type Definitions */

typedef	struct	adb
{
	LONG	screenaddr[2];	/* addresses of two screens for double buffering */
	BYTE	currentscreen; 	/* current active screen */
	BYTE	screenrate; 	/* display frequency (see enum below) */
	BYTE	screencount;
} ADB;

/* Display Frequencies */
enum {SINGLE_BUFFER, FRAME60HZ, FRAME30HZ, FRAME20HZ, FRAME15HZ,
	  FRAME12HZ, FRAME10HZ, FRAME5HZ=12, FRAME3HZ=20, FRAME2HZ=30,
	  FRAME1HZ=60};

/* Point Co-ordinates */
typedef	struct	point
{
	WORD	x;
	WORD	y;
} XY;

/* Window Co-ordinates */
typedef	struct	window
{
	WORD	x1;
	WORD	x2;
	WORD	y1;
	WORD	y2;
} WINDOW;

/* Colour values */
typedef	struct rgb
{
	UBYTE	red;
	UBYTE	green;
	UBYTE	blue;
} RGB;

/**** CPU ADDRESS SPACE ADDRESSING ****/

#define VIDEO_BASE (void*)0x30000000

/* video display memory: address offset - LINEAR ADDRESSING MODE */
#define CALYPSO32_LINEAR_RAM 			0x30000000

#define DISPLAY_LIST_OFFSET 		0x1f00000

#define TEXT_DISPLAY_LIST_OFFSET 	0x1f08000

/**** CALYPSO32 memory addressing: 32 Mbyte memory is fitted ****/
#define CALYPSO32_MEMORY_OFFSET		0x0

/* Pluto 6: Chip Select 3 Base Address*/
#define CS3							0x30000000

/* Video Memory */
#define Calypso32_Video_Memory		CS3

/* Cremson Address */
#define Cremson_Host_Base			CS3+0x1fc0000
#define Cremson_Display_Base		CS3+0x1fd0000
#define Cremson_Draw_Base			CS3+0x1ff0000

/**** CREMSON REGISTERS ****/

/* Host Interface Registers */
#define Cremson_DTC		Cremson_Host_Base+0x00		/* DMA Transfer Count Register 				*/
#define Cremson_DSU		Cremson_Host_Base+0x04		/* DMA Setup Register  						*/
#define Cremson_DRM		Cremson_Host_Base+0x05		/* DMA Request Mask Register  				*/
#define Cremson_DST		Cremson_Host_Base+0x06		/* DMA Status Register  					*/
#define Cremson_DTS		Cremson_Host_Base+0x08		/* DMA Transfer Stop Register				*/
#define Cremson_LSTA	Cremson_Host_Base+0x10		/* Display List Transfer Status Register	*/
#define Cremson_DRQ		Cremson_Host_Base+0x18		/* DMA Request Register  					*/
#define Cremson_IST		Cremson_Host_Base+0x20		/* Interrupt Status Register				*/
#define Cremson_IMASK	Cremson_Host_Base+0x24		/* Interrupt Mask Register  				*/
#define Cremson_SRST	Cremson_Host_Base+0x2c		/* Software Reset Register  				*/
#define Cremson_LSA		Cremson_Host_Base+0x40		/* Display List  Source Address Register	*/
#define Cremson_LCO		Cremson_Host_Base+0x44		/* Display List Count Register  			*/
#define Cremson_LREQ	Cremson_Host_Base+0x48		/* Display List Transfer Request Register  	*/

/* Graphics Memory Interface Registers */
#define Cremson_MMR		Cremson_Host_Base+0xfffc	/* Memory IF Mode Register  				*/

/* Display Control Registers */
#define Cremson_DCM		Cremson_Display_Base+0x00	/* Display Control Mode Register			*/
#define Cremson_DCE		Cremson_Display_Base+0x02	/* Display Controller Enable Register		*/

/* Horizontal Registers */
#define Cremson_HTP		Cremson_Display_Base+0x06	/* Hor Total Pixels Register  				*/
#define Cremson_HDP		Cremson_Display_Base+0x08	/* Hor Display Period Register  			*/
#define Cremson_HDB		Cremson_Display_Base+0x0a	/* Hor Display Boundary Register  			*/
#define Cremson_HSP		Cremson_Display_Base+0x0c	/* HSync Position Register  				*/
#define Cremson_HSW		Cremson_Display_Base+0x0e	/* HSync Width Register  					*/

/* Vertical Registers */
#define Cremson_VSW		Cremson_Display_Base+0x0f	/* Vsync Width Register  					*/
#define Cremson_VTR		Cremson_Display_Base+0x12	/* Vert Total Rasters Register				*/
#define Cremson_VSP		Cremson_Display_Base+0x14	/* Vsync Position Register  				*/
#define Cremson_VDP		Cremson_Display_Base+0x16	/* Vert Display Period Register 			*/

/* Window Registers */
#define Cremson_WX		Cremson_Display_Base+0x18	/* Window Position X Register  				*/
#define Cremson_WY		Cremson_Display_Base+0x1a	/* Window Position Y Register  				*/
#define Cremson_WW		Cremson_Display_Base+0x1c	/* Window Width Register  					*/
#define Cremson_WH		Cremson_Display_Base+0x1e	/* Window Height Register 					*/

/* Console Layer Registers */
#define Cremson_CM		Cremson_Display_Base+0x20	/* C Layer Mode 							*/
#define Cremson_COA		Cremson_Display_Base+0x24	/* C Layer Origin Address					*/
#define Cremson_CDA		Cremson_Display_Base+0x28	/* C Layer Display Address					*/
#define Cremson_CDX		Cremson_Display_Base+0x2c	/* C Layer Display Position X 				*/
#define Cremson_CDY		Cremson_Display_Base+0x2e	/* C Layer Display Position Y 				*/

/* Window Layer Registers */
#define Cremson_WM		Cremson_Display_Base+0x30	/* W Layer Mode 							*/
#define Cremson_WOA		Cremson_Display_Base+0x34	/* W Layer Origin Address					*/
#define Cremson_WDA		Cremson_Display_Base+0x38	/* W Layer Display Address					*/

/* Middle Layer Left Registers */
#define Cremson_MLM		Cremson_Display_Base+0x40	/* ML Layer Mode							*/
#define Cremson_MLOA0	Cremson_Display_Base+0x44	/* ML Layer Origin Address 0				*/
#define Cremson_MLDA0	Cremson_Display_Base+0x48	/* ML Layer Display Address 0				*/
#define Cremson_MLOA1	Cremson_Display_Base+0x4c	/* ML Layer Origin Address 1				*/
#define Cremson_MLDA1	Cremson_Display_Base+0x50	/* ML Layer Display Address 1				*/
#define Cremson_MLDX	Cremson_Display_Base+0x54	/* ML Layer Display Position X				*/
#define Cremson_MLDY	Cremson_Display_Base+0x56	/* ML Layer Display Position Y				*/

/* Middle Layer Right Registers */
#define Cremson_MRM		Cremson_Display_Base+0x58	/* MR Layer Mode							*/
#define Cremson_MROA0	Cremson_Display_Base+0x5c	/* MR Layer Origin Address 0				*/
#define Cremson_MRDA0	Cremson_Display_Base+0x60	/* MR Layer Display Address 0				*/
#define Cremson_MROA1	Cremson_Display_Base+0x64	/* MR Layer Origin Address 1				*/
#define Cremson_MRDA1	Cremson_Display_Base+0x68	/* MR Layer Display Address 1				*/
#define Cremson_MRDX	Cremson_Display_Base+0x6c	/* MR Layer Display Position X				*/
#define Cremson_MRDY	Cremson_Display_Base+0x6e	/* MR Layer Display Position Y				*/

/* Base Layer Left Registers */
#define Cremson_BLM		Cremson_Display_Base+0x70	/* BL Layer Mode							*/
#define Cremson_BLOA0	Cremson_Display_Base+0x74	/* BL Layer Origin Address 0				*/
#define Cremson_BLDA0	Cremson_Display_Base+0x78	/* BL Layer Display Address 0				*/
#define Cremson_BLOA1	Cremson_Display_Base+0x7c	/* BL Layer Origin Address 1				*/
#define Cremson_BLDA1	Cremson_Display_Base+0x80	/* BL Layer Display Address 1				*/
#define Cremson_BLDX	Cremson_Display_Base+0x84	/* BL Layer Display Position X				*/
#define Cremson_BLDY	Cremson_Display_Base+0x86	/* BL Layer Display Position Y				*/

/* Base Layer Right Registers */
#define Cremson_BRM		Cremson_Display_Base+0x88	/* BR Layer Mode							*/
#define Cremson_BROA0	Cremson_Display_Base+0x8c	/* BR Layer Origin Address 0				*/
#define Cremson_BRDA0	Cremson_Display_Base+0x90	/* BR Layer Display Address 0				*/
#define Cremson_BROA1	Cremson_Display_Base+0x94	/* BR Layer Origin Address 1				*/
#define Cremson_BRDA1	Cremson_Display_Base+0x98	/* BR Layer Display Address 1				*/
#define Cremson_BRDX	Cremson_Display_Base+0x9c	/* BR Layer Display Position X				*/
#define Cremson_BRDY	Cremson_Display_Base+0x9e	/* BR Layer Display Position Y				*/

/* Transparency Control Registers */
#define Cremson_CTC		Cremson_Display_Base+0xbc	/* C Layer Transparent Control				*/
#define Cremson_MRTC	Cremson_Display_Base+0xc0	/* MR Layer Transparent Control				*/
#define Cremson_MLTC	Cremson_Display_Base+0xc2	/* ML Layer Transparent Control 			*/

/* Palette Registers */
#define Cremson_CPal	Cremson_Display_Base+0x400	/* C Layer Palette Base 					*/
#define Cremson_MBPal	Cremson_Display_Base+0x800	/* C Layer Palette Base 					*/

/* Draw Control Registers */
#define Cremson_CTR		Cremson_Draw_Base+0x400		/* Control				 					*/

/*	Draw Mode Parameter Registers accessed via SetRegister Display List Command
	All register addresses are defined in BIG endian format.  Big to little endian conversion
	takes place on transfer from system to video memory
*/

#define CREMSON_MDR0	0x2004			/* Misc Mode Register						*/
#define CREMSON_MDR4	0x3004			/* BLT Mode Register 						*/
#define CREMSON_FBR		0x4004			/* Drawing Frame Base Register 				*/
#define CREMSON_XRES	0x4404			/* Drawing Frame Base Register 				*/
#define CREMSON_FC		0x8004			/* Foreground Colour 						*/
#define CREMSON_BC		0x8404			/* Background Colour 						*/

/* Triangle Draw Registers */

/* Line Draw Registers */

/* Pixel Plot Registers */

/* Rectangle Draw Registers */

/* Bit Registers */

/* Fast2DLine Draw Registers */

/* Fasts2DTriangle Draw Registers */

/* DisplayList FIFO Registers */


/* Display List Commands */
#define CREMSON_SET_REGISTER	0xF1
#define BLT_COPY_ALTP			0x0F
#define DRAW_BITMAPP     		0x0B

#define TOP_LEFT        	 	0x44
#define BLT_DRAW       		  	0x42
#define BITMAP         		 	0x43

#define INTERRUPT				0xFD	/* interrupt  display list command */
#define NOP						0xFF	/* nop display list command */

/* Layers */
#define C_LAYER				0x00
#define W_LAYER				0x01
#define ML_LAYER			0x02
#define MR_LAYER			0x03
#define BL_LAYER			0x04
#define BR_LAYER			0x05

/* Layer Enables*/
#define CONSOLE_ENABLE		0x01
#define WINDOW_ENABLE		0x02
#define MIDDLE_ENABLE		0x04
#define BASE_ENABLE			0x08

/* Layer Modes */
#define LAYER_FRAME0			0
#define LAYER_FRAME1			1
#define LAYER_FRAMEFLIP			2

/* Colour Modes */
#define INDIRECT_COLOR	0
#define DIRECT_COLOR	1


/* Update Sprite Mode */
#define TRANSPARENT_MODE 0x08

/* BLT Mode Bit Definitions*/
#define COLOR_EXPAND				0x80
#define COLOR_EXPAND_TRANSPARENT	0x08

/* BLT ROP Function Definitions */
#define SRCCLEAR        0x01
#define SRCCOPY         0x0d
#define SRCAND          0x05
#define SRCPAINT        0x6d

/* Layer Update Flag Bit Fields */
#define UPDATE_ENABLE	0x08

/****	Type Definitions */

/* driver configuraton populated in config.c at game level */
typedef	struct calypso32cfg
{
	const WORD* screenmode;
	const RGB*	palette;
} CALYPSO32CFG;

/* Display list parameters to write to processor */
typedef struct calypso32displaylist
{
	LONG list_source_address;	/* display List Source Address	*/
	LONG list_count;			/* display List Count			*/
} CALYPSO32DISPLAYLIST;


/* Layer properties */
typedef struct calypso32layer
{
	LONG mode;
	WORD transparent_colour;
	WORD xpos;
	WORD ypos;
	LONG orig_adr0;
	LONG orig_adr1;
	LONG disp_adr0;
	LONG disp_adr1;
	BYTE update_flag;
} CALYPSO32LAYER;

/* Font Properties */
typedef struct calypso32font
{
	const ULONG *start;
	LONG size;
	UBYTE width;
	UBYTE height;
	BYTE xoff;
	BYTE yoff;
} CALYPSO32FONT;


/* Character Properties */
typedef struct calypso32char
{
	ULONG start;
	UBYTE dx;
	UBYTE dy;
	UBYTE width;
	UBYTE height;
	BYTE xoff;
	BYTE yoff;
} CALYPSO32CHAR;


/****	Prototypes */

	#include	"video.p"

#endif
/*************************************************************************/
